/*
 * Copyright (C) 2003 Jelmer Vernooij <jelmer@samba.org>
 */

#include "kregeditview.h"

#include <qpainter.h>
#include <qlayout.h>

#include <kurl.h>

#include <klibloader.h>
#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>
#include <klistview.h>
#include <kio/netaccess.h>
#include <ktempfile.h>

kregeditView::kregeditView(QWidget *parent)
    : QWidget(parent),
      DCOPObject("kregeditIface")
{
    // setup our layout manager to automatically add our widgets
    QHBoxLayout *top_layout = new QHBoxLayout(this);
    top_layout->setAutoAdd(true);
    tree = new KListView(this, "tree");
    keys = new KListView(this, "keys");

    tree->addColumn("Tree");
    keys->addColumn("Type");
    keys->addColumn("Name");
    keys->addColumn("Data");

    if ((regf = nt_create_regf()) == NULL) {
        fprintf(stderr, "Could not create registry object: %s\n", strerror(errno));
    }

    connect(tree, SIGNAL(executed( QListViewItem * )), this, SLOT(viewVals(QListViewItem *)));
}

kregeditView::~kregeditView()
{
	nt_free_regf(regf);
}

void kregeditView::print(QPainter *p, int height, int width)
{
    // do the actual printing, here
    // p->drawText(etc..)
}

QString kregeditView::currentURL()
{
    return url.url();
}

void kregeditView::openURL(QString url)
{
    openURL(KURL(url));
}

void kregeditView::saveURL(QString url)
{
    saveURL(KURL(url));
}

void kregeditView::save()
{
    saveURL(this->url);
}

void kregeditView::openURL(const KURL& url)
{
  QString tmpfile;
  KIO::NetAccess::download( url, tmpfile );

  this->url = url;

  if (!nt_set_regf_input_file(regf, tmpfile)) {
	  KMessageBox::detailedSorry(this, "Can't open input file", strerror(errno), "Error opening file");
      return;
  }

  /* Now, open it, and bring it into memory :-) */
  if (nt_load_registry(regf) < 0) {
	  KMessageBox::detailedSorry(this, "Can't open input file", strerror(errno), "Error opening file");
      return;
  }

  KIO::NetAccess::removeTempFile( tmpfile );

  addKey();
}

void kregeditView::saveURL(const KURL &url)
{
  KTempFile tmpfile;

  if (!nt_set_regf_output_file(regf, tmpfile.name())) {
	  KMessageBox::detailedSorry(this, "Can't save file", strerror(errno), "Error saving file");
      return;
  }

  /* Now, open it, and bring it into memory :-) */

  if (nt_store_registry(regf) < 0) {
	  KMessageBox::detailedSorry(this, "Can't save file", strerror(errno), "Error saving file");
      return;
  }

  KIO::NetAccess::upload(tmpfile.name(), url);
}

void kregeditView::slotOnURL(const QString& url)
{
    emit signalChangeStatusbar(url);
}

void kregeditView::slotSetTitle(const QString& title)
{
    emit signalChangeCaption(title);
}

void kregeditView::addKey(KListViewItem *_parent, REG_KEY *_key)
{
	REG_KEY *key = _key;

	if(!key)key = regf->root;

	QString keyname(key->name);

	KListViewItem *listitem;

	if(_parent)listitem = new KListViewItem(_parent, keyname);
	else listitem = new KListViewItem(this->tree, keyname);

	if(!key->sub_keys)return;

	int i;
	for(i = 0; i< key->sub_keys->key_count; i++) {
		addKey(listitem, key->sub_keys->keys[i]);
	}
}

void kregeditView::viewVals( QListViewItem *item )
{
	/* Find the correct values in the list and add them to the right KListView */
	/* First, clear up all the current values listed there */
	keys->clear();

	if(!item) return; /* Huh? */

	/* Find out full name of this item */
	char *fullname = strdup("");

	QListViewItem *p = item;
	while(p) {
		char *new_fullname = NULL;
		asprintf(&new_fullname, "%s\\%s", p->text(0).ascii(), fullname);
		free(fullname);
		fullname = new_fullname;
		p = p->parent();
	}

	char *c = strrchr(fullname, '\\');
	*c = '\0';

	REG_KEY *key = nt_find_key_by_name(regf->root, fullname);

	if(!key) {
		printf("Huh!? the key %s suddenly went away!\n", fullname);
		free(fullname);
	}

	if(!key->values)return;

	int i;
	for(i = 0; i < key->values->val_count; i++) {
		VAL_KEY *val = key->values->vals[i];
		char data_asc[1024];
		data_to_ascii((unsigned char *)val->data_blk, val->data_len, val->data_type, data_asc, sizeof(data_asc) - 1);
		new KListViewItem(keys,	type_to_str(val->data_type),(val->name?val->name:"<No Name>"),	data_asc);
	}

	free(fullname);
}

#include "kregeditview.moc"
